#ifndef HTTPCLIENT_H
#define HTTPCLIENT_H

#include <ESP8266WiFi.h>
#include "HttpConstants.h"
#include "Dictionary.h"

namespace HttpClientHelper
{
  
}

class HttpClient : public WiFiClient, public HttpConstants
{
	public:
    enum ConnectionStatus { NO_ATTEMPT = 0, SUCCESSFUL, FAILED };
    
    HttpClient();
    HttpClient(const char *address, uint16_t port = 80);
    HttpClient(const char *address, const char *resource, uint16_t port = 80);

    HttpClient(const HttpClient &object);
    virtual ~HttpClient();

    boolean sendHttpQuery(const char *ressource, HttpRequestMethod method = HttpRequestMethod::GET, Dictionary<DictionaryHelper::StringEntity> *getData = NULL, Dictionary<DictionaryHelper::StringEntity> *postData = NULL);
    boolean sendHttpQuery(HttpRequestMethod method = HttpRequestMethod::GET, Dictionary<DictionaryHelper::StringEntity> *getData = NULL, Dictionary<DictionaryHelper::StringEntity> *postData = NULL);
    void keepAlive(boolean enabled);
    void setMaxRetries(int16_t retries);

    HTTP_CODE isReplyAvailable();
    
    uint16_t readHttpReply(uint8_t *buffer, uint32_t size);
	protected:
	private:
    boolean connectByHostOrIp();
    void sendUriWithGetParams(Dictionary<DictionaryHelper::StringEntity> *data);
    void sendPostData(Dictionary<DictionaryHelper::StringEntity> *data);
    void sendHeader(HttpMIMEType contentType = HttpMIMEType::UNKNOWN_MIME, uint64_t contentLength = 0, HttpVersion httpVersion = HttpVersion::HTTP_1_1);
    uint64_t computeBodyLength(Dictionary<DictionaryHelper::StringEntity> *data);
    
    ConnectionStatus _connectionStatus;
    char *_resource;
    char *_address;
    uint16_t _port;
    boolean _keepAlive;
    int16_t _maxRetries;
    uint16_t _retries;
    HTTP_CODE _httpCode;
    boolean _bodyReadyToRead;
};

#endif //HTTPCLIENT_H

/*
 * TCP status codes :
 * enum tcp_state {
  CLOSED      = 0,
  LISTEN      = 1,
  SYN_SENT    = 2,
  SYN_RCVD    = 3,
  ESTABLISHED = 4,
  FIN_WAIT_1  = 5,
  FIN_WAIT_2  = 6,
  CLOSE_WAIT  = 7,
  CLOSING     = 8,
  LAST_ACK    = 9,
  TIME_WAIT   = 10
};
 */
